;;;; -*- Mode: lisp; indent-tabs-mode: nil -*-
;;; checker.lisp --- Lisp version of checker.c (Red Book examples)
;;;
;;; Original C version contains the following copyright notice:
;;;   Copyright (c) 1993-1997, Silicon Graphics, Inc.
;;;   ALL RIGHTS RESERVED
;;;
;;; Ported to Lisp by Neil Gilmore (raito@raito.com)

(in-package #:cl-glut-examples)

(defclass checker-window (glut:window)
  ()
  (:default-initargs :pos-x 100 :pos-y 100 :width 250 :height 250 
                     :title "checker.lisp" :mode '(:single :rgb :depth)))

(defun make-check-image ()
  ; we can't use multi-dimensional arrays for textures
  (let ((ret-val (make-array '(16384) :element-type 'unsigned-byte 
                                      :initial-element 0)))
    (do ((y 0 (incf y))) 
        ((> y 63) ret-val) 
        (do ((x 0 (incf x))) 
            ((> x 63) nil) 
            (let ((base (* 4 (+ x (* y 64))))) 
                  (unless (eql (logand x 8) (logand y 8)) 
                    (setf (aref ret-val base) 255) 
                    (setf (aref ret-val (+ base 1)) 255)
                    (setf (aref ret-val (+ base 2)) 255)))))))

(defmethod glut:display-window :before ((w checker-window))
  (let ((tex-name (car (gl:gen-textures 1))))
    (gl:clear-color 0 0 0 0)
    (gl:shade-model :flat)
    (gl:enable :depth-test)
    (gl:pixel-store :unpack-alignment 1)
    (setf tex-name (car (gl:gen-textures 1)))
    (gl:bind-texture :texture-2d tex-name)
    (gl:tex-parameter :texture-2d :texture-wrap-s :repeat)
    (gl:tex-parameter :texture-2d :texture-wrap-t :repeat)
    (gl:tex-parameter :texture-2d :texture-mag-filter :nearest)
    (gl:tex-parameter :texture-2d :texture-min-filter :nearest)
    (gl:tex-image-2d :texture-2d 0 :rgba 64 64 0 :rgba :unsigned-byte (make-check-image))))

(defmethod glut:display ((w checker-window))
  (gl:clear :color-buffer-bit :depth-buffer-bit)
  (gl:enable :texture-2d)
  (gl:with-primitives :quads
    (gl:tex-coord 0.0 0.0) (gl:vertex -2.0 -1.0 0.0)
    (gl:tex-coord 0.0 1.0) (gl:vertex -2.0 1.0 0.0)
    (gl:tex-coord 1.0 1.0) (gl:vertex 0.0 1.0 0.0)
    (gl:tex-coord 1.0 0.0) (gl:vertex 0.0 -1.0 0.0)

    (gl:tex-coord 0.0 0.0) (gl:vertex 1.0 -1.0 0.0)
    (gl:tex-coord 0.0 1.0) (gl:vertex 1.0 1.0 0.0)
    (gl:tex-coord 1.0 1.0) (gl:vertex 2.41421 1.0 -1.41421)
    (gl:tex-coord 1.0 0.0) (gl:vertex 2.41421 -1.0 -1.41421))

  (gl:flush)
  (gl:disable :texture-2d))

(defmethod glut:reshape ((w checker-window) width height)
  (gl:viewport 0 0 width height)
  (gl:matrix-mode :projection)
  (gl:load-identity)
  (glu:perspective 60.0 (/ width height) 1.0 30.0)
  (gl:matrix-mode :modelview)
  (gl:load-identity)
  (gl:translate 0.0 0.0 -3.6))

(defmethod glut:keyboard ((w checker-window) key x y)
  (declare (ignore x y))
  (when (eql key #\Esc)
    (glut:destroy-current-window)))

(defun rb-checker ()
  (glut:display-window (make-instance 'checker-window)))
